]> AND Private Git Repository - loba.git/blobdiff - process.h
Logo AND Algorithmique Numérique Distribuée

Private GIT Repository
Use MSG_comm_waitall for communicator::flush(true).
[loba.git] / process.h
index c30b24374e0aed763bb626ed6d087350b1dbc548..24d4c395668b2d4209abaf6ddc5b4ebf8c315aa0 100644 (file)
--- a/process.h
+++ b/process.h
 #include <xbt/log.h>
 #include "communicator.h"
 #include "neighbor.h"
+#include "options.h"
 
 class process {
 public:
-    static double get_total_load_init() { return total_load_init; }
-    static double get_total_load_exit() { return total_load_exit; }
+    static double get_total_load_init()    { return total_load_init;    }
+    static double get_total_load_running() { return total_load_running; }
+    static double get_total_load_exit()    { return total_load_exit;    }
 
     process(int argc, char* argv[]);
     virtual ~process();
 
+    double get_comp() const                { return comp; }
+    double get_real_load() const           { return real_load; }
+
     int run();
 
 protected:
@@ -34,8 +39,14 @@ protected:
     pneigh_type pneigh;         // list of pointers to neighbors that
                                 // we are free to reorder
 
-    // Returns the sum of "to_send" for all neighbors.
-    double sum_of_to_send() const;
+    // Get and set current load, which may be real load, or expected
+    // load if opt::bookkeeping is true.
+    double get_load() const;
+    void set_load(double load);
+
+    // Register some amount of load to send to given neighbor.
+    void send(neighbor& nb, double amount);
+    void send(neighbor* nb, double amount) { send(*nb, amount); }
 
     // Calls neighbor::print(verbose, logp, cat) for each member of neigh.
     void print_loads(bool verbose = false,
@@ -48,8 +59,9 @@ protected:
                        xbt_log_category_t cat = _XBT_LOGV(default)) const;
 
 private:
-    static double total_load_init; // sum of neighbor loads at init
-    static double total_load_exit; // sum of neighbor loads at exit
+    static double total_load_init; // sum of process loads at init
+    static double total_load_running; // sum of loads while running
+    static double total_load_exit; // sum of process loads at exit
 
     typedef MAP_TEMPLATE<m_host_t, neighbor*> rev_neigh_type;
     neigh_type neigh;           // list of neighbors (do not alter
@@ -62,20 +74,20 @@ private:
     int data_close_pending;     // number of "close" messages to wait
                                 // on data channel
     bool close_received;        // true if we received a "close" message
-    bool may_receive;           // true if there remains neighbors to listen for
+    bool finalizing;            // true when finalize() is running
+
+    unsigned lb_iter;           // counter of load-balancing iterations
+    unsigned comp_iter;         // counter of computation iterations
 
-    unsigned iter;              // counter of iterations
+    double comp;                // total computing done so far (flops)
 
     double prev_load_broadcast; // used to ensure that we do not send
                                 // a same information messages
-    double load;                // current load
+    double real_load;           // current load
     double expected_load;       // expected load in bookkeeping mode
 
     // The load balancing algorithm comes here...
-    // Parameter "my_load" is the load to take into account for myself
-    // (may be load or expected load).
-    // Returns the total load sent to neighbors.
-    virtual double load_balance(double my_load);
+    virtual void load_balance();
 
     // Virtually do some computation
     void compute();
@@ -83,11 +95,17 @@ private:
     // Send procedures, with helpers for bookkeeping mode or not
     void send1_no_bookkeeping(neighbor& nb);
     void send1_bookkeeping(neighbor& nb);
-    void send();
+    void send_all();
 
-    // Receive procedure: wait (or not) for a message to come.
-    enum recv_wait_mode { NO_WAIT = 0, WAIT, WAIT_FOR_CLOSE };
-    void receive(recv_wait_mode wait);
+    // Returns true if there remains neighbors to listen for
+    bool may_receive() const {
+        return ctrl_close_pending || data_close_pending;
+    }
+
+    // Receive procedure
+    // Parameter "timeout" may be 0 for non-blocking operation, -1 for
+    // infinite waiting, or any positive timeout.
+    void receive(double timeout);
 
     // Finalize sends a "close" message to each neighbor and wait for
     // all of them to answer.
@@ -95,6 +113,24 @@ private:
     void finalize();
 };
 
+inline
+double process::get_load() const
+{
+    if (opt::bookkeeping)
+        return expected_load;
+    else
+        return real_load;
+}
+
+inline
+void process::set_load(double load)
+{
+    if (opt::bookkeeping)
+        expected_load = load;
+    else
+        real_load = load;
+}
+
 #endif // !PROCESS_H
 
 // Local variables: